;
;   APPLICATION in 800x 600 resolution
;   2D rotation
;   Compile with FASM for Menuet
;   author : macgub
;   email : macgub3@wp.pl
TIMEOUT EQU 1


use32

               org    0x0

               db     'MENUET01'              ; 8 byte id
               dd     0x01                    ; header version
               dd     START                   ; start of code
               dd     I_END                   ; size of image
               dd     0x300000                ; memory for app
               dd     0xffff0                 ; esp
               dd     0x0 , 0x0               ; I_Param , I_Icon

START:                          ; start of execution

     call draw_window

still:

    mov  eax,23                 ; wait here for event
    mov  ebx,TIMEOUT
    int  0x40

    cmp  eax,1                  ; redraw request ?
    je   red
    cmp  eax,2                  ; key in buffer ?
    je   key
    cmp  eax,3                  ; button in buffer ?
    je   button

    jmp  noclose

  red:                          ; redraw
    call draw_window
    jmp  still

  key:                          ; key
    mov  eax,2                  ; just read it and ignore
    int  0x40
    jmp  still

  button:                       ; button
    mov  eax,17                 ; get id
    int  0x40

    cmp  ah,1                   ; button id=1 ?
    jne  noclose

    mov  eax,-1                 ; close this program
    int  0x40
  noclose:

;draw bar
; mov ebx, 5*65536 + 390
; mov ecx, 20*65536 + 375
; mov edx, 0x00000000
; mov eax, 13
; int 0x40
 call calculate_deg
 call clear_rotated_img_mem
 mov eax,0                  ;main loop
oop:
 mov ebx,0
oop1:
 push eax
 push ebx
    mov [x],eax
    mov [y],ebx
    cmp [img_load],1
    je img_loaded
    call getpixel_to_mem
    img_loaded:
    call getcol         ; from memory
    call rotate
    call putpixel_to_mem
   ; call putpixel
 pop ebx
 pop eax
 inc ebx
 cmp ebx,255
 jne oop1
 inc eax
 cmp eax,255
 jne oop
 mov [img_load],1
 call put_image

 jmp  still

clear_rotated_img_mem:
 mov eax,image
 add eax,256*256*4
 mov ecx,96*384*3 ;(384*384*3) div 4
 xor ebx,ebx
 cleo:
 mov [eax],ebx
 add eax,4
 loop cleo
 ret

putpixel_to_mem:
 mov eax,[y]
 add eax,50
 mov ebx,eax
 shl eax,8
 shl ebx,7
 add eax,ebx   ; y*384
 add eax,[x]
 add eax,55
 mov ebx,eax
 shl eax,1
 add eax,ebx ;(y*384 + x)*3
 add eax,image
 add eax,256*256*4 ; 256 * 256*4 - image size; eax - rotated image pix addres
 mov ebx,[col]
 or [eax],ebx
 ret
put_image:
 mov ebx,image
 add ebx,256*256*4
 mov ecx,384 * 65536 + 377
 mov edx,5 * 65536 + 20
 mov eax,7
 int 0x40
 ret
getcol:
 mov ebx,[y]
 shl ebx,8
 add ebx,[x]
 shl ebx,2
 add ebx,image
 mov eax,[ebx]
 mov [col],eax
 ret

getpixel_to_mem:
 mov ebx,[y]
 mov eax,ebx
 mov ecx,ebx
 shl ebx,9
 shl eax,8
 shl ecx,5
 add ebx,eax
 add ebx,ecx
 add ebx,[x]
 mov eax,35
 int 0x40
 push eax
 mov eax,[y]
 shl eax,8
 add eax,[x]
 shl eax,2
 add eax,image
 pop edx
 mov [eax],edx
 ret

;putpixel:
; mov eax,1
; mov ebx,[x]
; add ebx,80
; mov ecx,[y]
; add ecx,80
; mov edx,[col]
; int 0x40
; ret
rotate:
 mov eax,[y]
 sub eax,[yo]
 mov [ysub],eax
 mov eax,[x]
 sub eax,[xo]
 mov [xsub],eax

 fld [singamma]
 fimul [ysub]
 fld [cosgamma]
 fimul [xsub]
 faddp ;st1,st
 fiadd [xo]
 fistp [x]
 fld [cosgamma]
 fimul [ysub]
 fld [singamma]
 fimul [xsub]
 fchs
 faddp ;st1,st
 fiadd [yo]
 fistp [y]
 ret

calculate_deg:
 mov eax,[deg_counter]
 inc eax
 cmp eax,360
 jne calc
 xor eax,eax
 calc:
 mov [deg_counter],eax
 finit
 fldpi
 fidiv [div_deg]
 fimul [deg_counter]
 fsincos
 fstp [singamma]
 fstp [cosgamma]
 ret
;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************


draw_window:


    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,1                     ; 1, start of draw
    int  0x40

                                   ; DRAW WINDOW
    mov  eax,0                     ; function 0 : define and draw window
    mov  ebx,100*65536+400         ; [x start] *65536 + [x size]
    mov  ecx,100*65536+400         ; [y start] *65536 + [y size]
    mov  edx,0x02000000            ; color of work area RRGGBB,8->color gl
    mov  esi,0x805080d0            ; color of grab bar  RRGGBB,8->color gl
    mov  edi,0x005080d0            ; color of frames    RRGGBB
    int  0x40

                                   ; WINDOW LABEL
    mov  eax,4                     ; function 4 : write text to window
    mov  ebx,8*65536+8             ; [x start] *65536 + [y start]
    mov  ecx,0x10ddeeff            ; font 1 & color ( 0xF0RRGGBB )
    mov  edx,labelt                ; pointer to text beginning
    mov  esi,labellen-labelt       ; text length
    int  0x40

                                   ; CLOSE BUTTON
    mov  eax,8                     ; function 8 : define and draw button
    mov  ebx,(400-19)*65536+12     ; [x start] *65536 + [x size]
    mov  ecx,5*65536+12            ; [y start] *65536 + [y size]
    mov  edx,1                     ; button id
    mov  esi,0x6688dd              ; button color RRGGBB
    int  0x40


    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,2                     ; 2, end of draw
    int  0x40

    ret

; DATA AREA
yo dd 128
xo   dd 128
xsub dd ? ;integer
ysub dd ?
div_deg dd 45 ;180
deg_counter dd 0 ;integer
cosgamma dd ? ;real
singamma dd ? ;real
xresolution dd 800
col dd ?
x dd ?
y dd ?
img_load db 0

labelt:
     db   '2D ROTATION'
labellen:
image:
I_END:




